      program tracers
      use mod_tracer ! HYCOM post-processing synthetic float interface
      use mod_za     ! HYCOM array I/O interface
c
      implicit none
c
c --- post-processing lagrangian floats from a sequence of HYCOM 2.0
c --- archive files.
c
c instantaneous and mean archive names
      character*120    flnmi,flnmm  
      logical          lexist,lfatal,trcout
      integer          iexpt,yrflag,ka,l
      real             depthi(0:1,0:1,0:99),duk,dukm1,dvk,dvkm1
      real             dummy
      double precision time_ai,time_ao,timei_ai,timei_ao,time1,time2
      real             eps,time,delt
      real             sumtr,sumdp
      real             q1,q2
      integer          mxlflg
      integer          t1,t2
c
      real amin,amax
      integer imin,jmin,imax,jmax
      integer iord
      integer kmixl
      real, allocatable, dimension (:,:) :: sumhrdv
      call xcspmd
      call zaiost
c
      lp=6
      onecm=0.01*onem
      tencm=0.10*onem
c
      pi = 4.0*atan(1.0)
      radian=pi/180.0
c
c --- 'flnm_ai'  = name of first input archive file
c --- 'flnm_ao'  = name of final input archive file
c --- 'iexpt '   = experiment number x10  (000=from archive file)
c --- 'yrflag'   = days in year flag (0=360J16,1=366J16,2=366J01,3=actual)
c --- 'archfq'   = no. of hours between archive input
c --- 'archin'   = no. of hours between time-interpolated archive input
c --- 'idm   '   = longitudinal array size
c --- 'jdm   '   = latitudinal  array size
c --- 'kdm   '   = number of layers
c
c --- initial and final mean archives
      read(5,*) flnm_ai
      write(6,*)'flnm_ai  ',trim(flnm_ai)
      read(5,*) flnm_ao
      write(6,*)'flnm_ao  ',trim(flnm_ao)
c --- initial and final instantaneous archives
      read(5,*) flnmi_ai
      write(6,*)'flnmi_ai  ',trim(flnmi_ai)
      read(5,*) flnmi_ao
      write(6,*)'flnmi_ao  ',trim(flnmi_ao)
      call blkini(iexpt, 'iexpt ')
      call blkini(yrflag,'yrflag')
      call blkinr(archfq,
     &           'archfq','("blkinr: ",a6," =",f11.4," hours")')
      call blkinr(archin,
     &           'archin','("blkinr: ",a6," =",f11.4," hours")')
      archfq=archfq/24.0
      archin=archin/24.0
      nintrp=0
      if (archfq.ne.archin) then
        nintrp=nint(archfq/archin)
        if (archfq/archin.ne.float(nintrp)) then
          stop 'archfq must be an integer multiple of archin'
        endif
      endif
      delt=archfq*86400.
      write(6,*)'delt as archfq in seconds', delt
      call blkini(ii,    'idm   ')
      call blkini(jj,    'jdm   ')
      call blkini(kdm,   'kdm   ')
      call blkini(kk,    'kk    ')
      if     (ii.ne.idm .or. jj.ne.jdm) then
        write(lp,*)
        write(lp,*) 'error - wrong idm or jdm (should be:',idm,jdm,')'
        write(lp,*)
        call flush(lp)
        stop 
      endif
c --- 'thbase'   = reference sigma
c --- 'ntracr'   = number of tracers
c --- 'tracin'   = 0: initialze to 0; 1: from restart_tracr_in; 2: from relax.tracer.a,b
c --- 'mxlflg'   = 0 no extra mixing in mixed layer; 1: instantaneous homog of mixed layer.
c --- 'restfq'   tracer restart frequency
c#### not implemented c --- 'trcflg'   = 0 surface tracer
c#####not implememted c --- 'forflg'   = 0 no forcing
c --- 'iord  '   = 1:simple donnor cell; 2: complete with antidiffusive fluxes
c --- 'itest ' = i for printout
c --- 'jtest ' = j for printout
      call blkinr(thbase,
     &           'thbase','("blkinr: ",a6," =",f11.4," days")')
      call blkini(ntracr,'ntracr')
      call blkini(tracin,'tracin')
      call blkini(mxlflg,'mxlflg')
      call blkinr(restfq,
     &           'restfq','("blkinr: ",a6," =",f11.4," hours")')
      restfq=restfq/24.
      call blkini(iord  ,'iord  ')
      call blkini(itest, 'itest ')
      call blkini(jtest, 'jtest ')
c
c --- array allocation
c
      call tracer_alloc
      allocate (sumhrdv(ii,jj))
c
c --- land masks.
c
      call geopar
c
c --- first and last model days from the input archives.
c
      call getdat_time(flnm_ai,time_ai,iexpt)
      write(6,*)'mean initial archive time', time_ai
      call getdat_time(flnm_ao,time_ao,iexpt)
      write(6,*)'mean final archive time', time_ao
      call getdat_time(flnmi_ai,timei_ai,iexpt)
      write(6,*)'intantaneous initial archive time', timei_ai
      call getdat_time(flnmi_ao,timei_ao,iexpt)
      write(6,*)'intantaneous final archive time', timei_ao
      eps=0.002
      if((abs(time_ai-timei_ai-archfq/2)).gt.eps) then
        write(6,*)'error, initial instantaneous and mean 
     &             archive times not consistent'
        stop
      endif
      write(6,*)'time_ao', time_ao
      write(6,*)'timei_ao',timei_ao
      write(6,*)'archfq',archfq
      write(6,*)'time_ao-timei_ao+archfq/2',time_ao-timei_ao+archfq/2
      if((abs(time_ao-timei_ao+archfq/2)).gt.eps) then
        write(6,*)'error, final instantaneous and mean 
     &             archive times not consistent'
        stop
      endif
c
      narch = nint((time_ao-time_ai)/archfq) + 1
c
      write(lp,900) flnm_ai,flnm_ao
  900 format('first and last archives files:'//a120/a120)
      write(lp,910) narch,time_ai,time_ao
  910 format(/'number of archives, start and end times:'/i6,
     &       1p,2e16.8)
c
c --- initialize the tracer to zero or from a restart file
c --- 
      time=time_ai
      write(6,*)'tracer_init'
      call tracer_init(tracin,ntracr,time)
      do ktr=1,ntracr
      write(6,*)'tracer ktr', ktr,itest,jtest
      do k=1,kk
          write(6,'(i3,f8.4)')k, tracer(itest,jtest,k,ktr) 
      enddo
      enddo

      flnmi=flnmi_ai
      flnmm=flnm_ai
c
c --- each time step needs 2 instantaneous archives for delta dp in time, 
c --- and one mean archive
c ---
c --- main loop
c ---
      do iar=1,narch

c --- dpii instantaneous initial dp
c --- dpio instantaneous end of interval dp
c --- read initial instantaneous archive dp and save it in meters
        if(iar.eq.1)then
          time2=timei_ai+(iar-1)*archfq
          call getdat_name(flnmi,time2,yrflag)
          write(lp,'(a,i4,f10.2,/a)') 'iar,time2,flnmi = ',
     &                               iar,time2,flnmi(1:len_trim(flnmi))
          call flush(lp)
        endif

c
c ---   end of interval instantaneous archive dp
        time2=timei_ai+(iar)*archfq
        write(6,*)'calling getdat_name'
        call getdat_name(flnmi,time2,yrflag)
        write(lp,'(a,i4,f10.2,/a)') 'iar,time2,flnmi = ',
     &                               iar,time2,flnmi(1:len_trim(flnmi))
        call flush(lp)
c
c ---   mean archive: read dp,u,v
c ---                 read also dpmixl if homogenization wanted
        time2=time_ai+(iar-1)*archfq
        write(6,*)'calling getdat_name'
        call getdat_name(flnmm,time2,yrflag)
        write(lp,'(a,i4,f10.2,/a)') 'iar,time2,flnmm = ',
     &                               iar,time2,flnmm(1:len_trim(flnmm))
        call flush(lp)

        time2=timei_ai+(iar)*archfq
        write(6,*)'calling getdat_name'
        call getdat_name(flnmi,time2,yrflag)
        write(lp,'(a,i4,f10.2,/a)') 'iar,time2,flnmi = ',
     &                               iar,time2,flnmi(1:len_trim(flnmi))
        write(6,*)flnmi
        call flush(lp)
        flnm_o='restart_out_'//flnmi(11:21)
        write(6,('(a)') )flnm_o
      call tracer_restart(time2,flnm_o)
      enddo  ! iar
      time2=timei_ai+narch*archfq
      call getdat_name(flnm_o,time2,yrflag)
      flnm_o='restart_out_'//flnm_o(11:21)
      call tracer_restart(time2,flnm_o)
c
c
      stop
c
      end
      subroutine pr_5x5(array,idm,jdm,iz,jz,offset,scale,what)
c
c --- (sames as prt9x9 except that array dimensions are passed as arguments)
c
c --- write 5 x 5 point cluster of 'array' values centered on (iz,jz).
c --- the printed numbers actually represent (array(i,j) + offset) * scale
c
      implicit none
      integer,intent(IN) :: idm,jdm,iz,jz
      real   ,intent(IN) :: array(idm,jdm),scale,offset
      character what*(*),text*12
      integer i,j,jwrap
 100  format(a12,5i13)
 101  format(i10,3x,5f13.5)
c
      text='            '
      text(1:min(12,len_trim(what)))=what(1:min(12,len_trim(what)))
      write (*,100) text,(i,i= iz-2,iz+2)
      write (*,101) (j,(scale*(array(i,j)+offset),
     . i=iz-2,iz+2),j=jz+2,jz-2,-1)
      return
      end
      subroutine aminmax(ip,array,idm,jdm,amin,amax,imin,jmin,imax,
     .                   jmax,spval)
      implicit none
      real array,amin,amax,spval
      integer ip, idm,jdm,imin,jmin,imax,jmax,i,j
      dimension array(idm,jdm),ip(idm,jdm)
      amin=1.e40
      amax=-1.e40 
      do j=1,jdm
        do i=1,idm
          if(ip(i,j).eq.1) then
            if(array(i,j).lt.amin.and.array(i,j).ne.spval) then
              amin=array(i,j)
              imin=i
              jmin=j
            endif
            if(array(i,j).gt.amax.and.array(i,j).ne.spval) then
              amax=array(i,j)
              imax=i
              jmax=j
            endif
          endif
        enddo
      enddo
      write(6,*)'min,max', amin,imin,jmin, amax,imax,jmax
      return
      end
